/* File: JOYSTICK.CPP
** Description:
**   Defines routines for reading the state of the joystick and
**   sensing if a joystick is attached to the game port.
*/

#include <stdio.h>
#include <dos.h>
#include <conio.h>

#include "joy.h"
#include "dpmi.h"

#define JOYSTICK_PORT (0x201)

/*
        Function: JOY_Init
        Description:
                Tries to determine which joystick axes are available to
                be read.  The routine works by repeatedly reading the
		joystick port location for a very long time.  Those
		axes that time out before we are done reading are assumed
		to be present.  Axes that don't time out by the time that
		we're done are assumed to not be present.  Note that on
		a very fast machine, this routine could be fooled.
*/

TJOY_state JOY_state;

#define JOY_INIT_LIMIT 30000

int JOY_init (void)
{
  int portval;
  int limit;
  int exitCode;

  /* initialize counters */
  JOY_state.a.axisX = JOY_state.a.axisY = 0;
  JOY_state.b.axisX = JOY_state.b.axisY = 0;
  limit = JOY_INIT_LIMIT;

  /* don't let interrupts happen while we're reading */
  DPMI_disableInt ();

  /* start the timers -- data value of write is unimportant */
  outp(JOYSTICK_PORT, 0x00);

  /* do until interesting bits all drop to zero */
  while (limit) {
	  portval = inp (JOYSTICK_PORT);
	  JOY_state.a.axisX += portval & 1;
	  portval >>= 1;
	  JOY_state.a.axisY += portval & 1;
	  portval >>= 1;
	  JOY_state.b.axisX += portval & 1;
	  portval >>= 1;
	  JOY_state.b.axisY += portval & 1;
	  limit--;
  }

  DPMI_enableInt ();
  /* enable interrupts */

  JOY_state.a.installed= JOY_state.b.installed = 0;
  if ((JOY_state.a.axisX != JOY_INIT_LIMIT) && (JOY_state.a.axisX != 0))
  {
	  JOY_state.mask |= 1;
	  JOY_state.a.installed= JOY_DETECTED;
  }
  if ((JOY_state.a.axisY != JOY_INIT_LIMIT) && (JOY_state.a.axisY != 0))
  {
	  JOY_state.mask |= 2;
	  JOY_state.a.installed= JOY_DETECTED;
  }
  if ((JOY_state.b.axisX != JOY_INIT_LIMIT) && (JOY_state.b.axisX != 0))
  {
	  JOY_state.mask |= 4;
	  JOY_state.b.installed= JOY_DETECTED;
  }
  if ((JOY_state.b.axisY != JOY_INIT_LIMIT) && (JOY_state.b.axisY != 0))
  {
	  JOY_state.mask |= 8;
	  JOY_state.b.installed= JOY_DETECTED;
  }

  JOY_state.a.minX = -1;
  JOY_state.a.minY = -1;
  JOY_state.a.maxX = 0;
  JOY_state.a.maxY = 0;
  JOY_state.b.minX = -1;
  JOY_state.b.minY = -1;
  JOY_state.b.maxX = 0;
  JOY_state.b.maxY = 0;

  exitCode = JOY_state.a.installed+ (JOY_state.b.installed<< 1);
  if (exitCode == 0)
    exitCode = JOY_NOTDETECTED;
  return exitCode;
}

/*
	Function: JOY_Read
	Description:
		Reads the current values of the joystick positions and
		button states.  axis values are returned in the  structure
		variables AaxisX/AY/BX/BY and button states in
		the structure variables Abutton1/A2/B1/B2.  Note that
		the structure variable Mask controls which axes of
		which joysticks (either A or B) should be waited for.
		If the B joystick, for instance, is not present, and
		Mask indicates that BaxisY should be waited
		for, the routine will wait an excessively long time.
		The routine SenseJoysticks, above, can be used to try to
		automatically determine which axes of which joysticks
		are present.  Note that for joystick axes not present,
		AaxisX/AY/BX/BY are set to garbage values.
*/
void JOY_read (void)
{
  int portval;
  int limit;

  /* initialize counters */
  JOY_state.a.axisX = JOY_state.a.axisY = 0;
  JOY_state.b.axisX = JOY_state.b.axisY = 0;
  limit = JOY_INIT_LIMIT ;

  /* don't let interrupts happen while we're reading */
  DPMI_disableInt ();

  /* start the timers -- data value of write is unimportant */
  outp(JOYSTICK_PORT, 0x00);

  /* do until interesting bits all drop to zero */
  while (((portval = inp(JOYSTICK_PORT)) & JOY_state.mask) && limit) {
	  JOY_state.a.axisX += portval & 1;
	  portval >>= 1;
	  JOY_state.a.axisY += portval & 1;
	  portval >>= 1;
	  JOY_state.b.axisX += portval & 1;
	  portval >>= 1;
	  JOY_state.b.axisY += portval & 1;
	  limit--;
  }

  /* recognize interrupts again */
  DPMI_enableInt ();

  /* update button variables -- note these are inverted from the port */
  (portval & 0x10) ? (JOY_state.a.button1 = 0) : (JOY_state.a.button1 = 1);
  (portval & 0x20) ? (JOY_state.a.button2 = 0) : (JOY_state.a.button2 = 1);
  (portval & 0x40) ? (JOY_state.b.button1 = 0) : (JOY_state.b.button1 = 1);
  (portval & 0x80) ? (JOY_state.b.button2 = 0) : (JOY_state.b.button2 = 1);
}

void JOY_moveUpperLeftA ()
{
  JOY_read ();

  if (JOY_state.a.minX > JOY_state.a.axisX)
  {
    JOY_state.a.minX = JOY_state.a.axisX;
  }
  if (JOY_state.a.minY > JOY_state.a.axisY)
  {
    JOY_state.a.minY = JOY_state.a.axisY;
  }
}

void JOY_moveUpperLeftB ()
{
  JOY_read ();

  if (JOY_state.b.minX > JOY_state.b.axisX)
  {
    JOY_state.b.minX = JOY_state.b.axisX;
  }
  if (JOY_state.b.minY > JOY_state.b.axisY)
  {
    JOY_state.b.minY = JOY_state.b.axisY;
  }
}

void JOY_moveLowerRightA ()
{
  JOY_read ();

  if (JOY_state.a.maxX < JOY_state.a.axisX)
  {
    JOY_state.a.maxX = JOY_state.a.axisX;
  }
  if (JOY_state.a.maxY < JOY_state.a.axisY)
  {
    JOY_state.a.maxY = JOY_state.a.axisY;
  }
}

void JOY_moveLowerRightB ()
{
  JOY_read ();

  if (JOY_state.b.maxX < JOY_state.b.axisX)
  {
    JOY_state.b.maxX = JOY_state.b.axisX;
  }
  if (JOY_state.b.maxY < JOY_state.b.axisY)
  {
    JOY_state.b.maxY = JOY_state.b.axisY;
  }
}


int JOY_positionA (void)
{
  int pos = 0;
  int sx = JOY_state.a.maxX-JOY_state.a.minX;
  int sy = JOY_state.a.maxY-JOY_state.a.minY;

  if (JOY_state.a.axisX > (JOY_state.a.minX+(sx*3)/4))
  {
    pos |= JOY_RIGHT;
  }
  else
  {
    if (JOY_state.a.axisX < (JOY_state.a.minX+(sx*1)/4))
    {
      pos |= JOY_LEFT;
    }
  }

  if (JOY_state.a.axisY > (JOY_state.a.minY+(sy*3)/4))
  {
    pos |= JOY_DOWN;
  }
  else
  {
    if (JOY_state.a.axisY < (JOY_state.a.minY+(sy*1)/4))
    {
      pos |= JOY_UP;
    }
  }

  return pos;
}
